"use strict";

exports.__esModule = true;
exports.default = void 0;

var _ramda = require("ramda");

var _util = require("./util");

/**
 * Combines two values of the same type (if it makes sense to combine them).
 * Numbers are summarized, strings and arrays are concatenated together, and
 * true objects are merged (the second value merged on top of the first).
 * In any other case only the first value is returned.
 *
 * @func
 * @sig a -> a -> a
 * @param {*} val1 The base value to be combined with
 * @param {*} val2 The value to combine
 * @returns {*} If the values are of the same type, this represents the combined
 * value of the two of them. Otherwise only the first value is returned
 */
var _default = (0, _ramda.curry)(function (val1, val2) {
  return (0, _ramda.cond)([[(0, _ramda.all)((0, _ramda.is)(Number)), _ramda.sum], [(0, _ramda.all)((0, _ramda.is)(String)), (0, _ramda.join)('')], [(0, _ramda.all)((0, _ramda.is)(Array)), _ramda.flatten], [(0, _ramda.all)(_util.isPlainObj), _ramda.mergeAll], [_ramda.T, (0, _ramda.always)(val1)]])([val1, val2]);
});

exports.default = _default;