"use strict";

exports.__esModule = true;
exports.default = void 0;

var _ramda = require("ramda");

var _util = require("./util");

var makeSpec = function makeSpec(transforms) {
  var spec = (0, _ramda.map)((0, _ramda.ifElse)(_util.isPlainObj, makeSpec, _util.alwaysFunction), (0, _util.safeSpecTransforms)(transforms));
  return (0, _ramda.curryN)((0, _ramda.reduce)(_ramda.max, 0, (0, _ramda.pluck)('length', (0, _ramda.values)(spec))), function () {
    for (var _len = arguments.length, allArgs = new Array(_len), _key = 0; _key < _len; _key++) {
      allArgs[_key] = arguments[_key];
    }

    return (0, _ramda.map)(function (fn) {
      return (0, _ramda.apply)(fn, allArgs);
    }, spec);
  });
};
/**
 * A port over of [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec),
 * but with a few changes you may or may not see as necessary.
 * (1) Properly curried so that you can optionally provide args as (spec, input) rather than only as (spec)(input)
 * (2) It wraps each function in a tryCatch that logs the key name of the transform function that fails.
 *
 * Using applySpec() in the wild and at-scale proved to be one of the hardest Ramda functions to debug.
 * Limiting its functionality and adding console logging doesn't sound like a necessary thing,
 * but when working with Ramda on a full team it may help prevent devs from getting so frustrated
 * that they vote to abandon the library while they're still learning the functional programming paradigm.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which
 * expect the entire object to be fed in as input.
 * This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object transformed to contain only the props named in the spec
 * (with the corresponding transforms applied at that key)
 */


var _default = (0, _ramda.curry)(function (transforms, input) {
  return makeSpec(transforms)(input);
});

exports.default = _default;