"use strict";

exports.__esModule = true;
exports.default = void 0;

var _ramda = require("ramda");

var _combine = _interopRequireDefault(require("./combine"));

var _mapSpec = _interopRequireDefault(require("./mapSpec"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Similar to [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec), however
 * the spec will be applied and then merged into the original input.
 * Ramda's applySpec() will omit the original input, returning an object containing _only_
 * the props named in your spec, but this function here will apply the spec
 * and then merge back it into the original input.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which
 * expect the entire object to be fed in as input.
 * This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object with all the transform functions applied at the named keys
 */
var _default = (0, _ramda.curry)(function (spec, value) {
  return (0, _ramda.converge)(_combine.default, [_ramda.identity, (0, _mapSpec.default)(spec)])(value);
});

exports.default = _default;