"use strict";

exports.__esModule = true;
exports.default = void 0;

var _ramda = require("ramda");

var _util = require("./util");

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but any
 * primitive value or array in the transform spec is wrapped in a function that
 * always returns that value. Any functions already on the transform spec are
 * applied at the prop-level (like evolve() always does) and any objects on the
 * transform spec call the evolver recursively (again, normal behavior for
 * evolve()).
 * In other words, the only difference between this and Ramda's
 * evolve() function is that primitives and array vales are wrapped in an
 * always() function before the spec is fed to evolve() rather than being
 * omitted from the final output.
 *
 * @func
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object with transform functions or nested transform
 * functions or primitive vales to be wrapped with a function.
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms
 * applied to it
 */
var evolveSpec = (0, _ramda.curry)(function (transforms, input) {
  var result = {};
  var spec = (0, _util.safeSpecTransforms)(transforms);
  var object = (0, _util.objectify)(input);
  (0, _ramda.keys)(object).forEach(function (key) {
    result[key] = (0, _ramda.cond)([[(0, _ramda.is)(Function), (0, _ramda.applyTo)(object[key])], [_util.isPlainObj, evolveSpec(_ramda.__, object[key])], [_ramda.T, function (val) {
      return (0, _ramda.has)(key, spec) ? val : object[key];
    }]])(spec[key]);
  });
  return result;
});
var _default = evolveSpec;
exports.default = _default;
module.exports = exports.default;